from django.shortcuts import render, redirect, get_object_or_404
from django.core.paginator import Paginator
from django.http import HttpResponse
import pandas as pd
from branch.models import CompanyBranch
from django.contrib import messages
from django.http import HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.db import IntegrityError

@login_required
def branches(request):
    if request.user.is_admin_user or request.user.is_super_user:
        try:
            branches_list = CompanyBranch.objects.all()
            paginator = Paginator(branches_list, 10)

            page_number = request.GET.get('page')
            page_obj = paginator.get_page(page_number)

            # Export to Excel functionality
            if 'export' in request.GET:
                data = []
                for branch in branches_list:
                    data.append({
                        'Branch Location': branch.branch_location,
                        'Branch Manager': branch.branch_manager,
                        'Created At': branch.created_at,
                        'Updated At': branch.updated_at,
                    })
                df = pd.DataFrame(data)
                response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
                response['Content-Disposition'] = 'attachment; filename=branches.xlsx'
                df.to_excel(response, index=False)
                return response

            data = {
                "page_obj": page_obj
            }
            return render(request, "backoffice/branch/branches.html", data)
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('backoffice:branches')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_branch(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.method == 'POST':
            try:
                branch_name = request.POST.get('branch_name')
                contact_person = request.POST.get('contact_person')
                address = request.POST.get('address')
                city = request.POST.get('city')
                state = request.POST.get('state')
                postal_code = request.POST.get('postal_code')
                country = request.POST.get('country')
                phone = request.POST.get('phone')
                email = request.POST.get('email')

                new_branch = CompanyBranch(
                    branch_name=branch_name,
                    contact_person=contact_person,
                    address=address,
                    city=city,
                    state=state,
                    postal_code=postal_code,
                    country=country,
                    phone=phone,
                    email=email
                )
                new_branch.save()
                messages.success(request, 'Branch added successfully.')
                return redirect('backoffice:branches')
            except IntegrityError as e:
                messages.error(request, f"Data integrity error: {str(e)}")
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")

        return render(request, "backoffice/branch/add_branch.html")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def edit_branch(request, branch_id):
    if request.user.is_admin_user or request.user.is_super_user:
        try:
            branch = get_object_or_404(CompanyBranch, id=branch_id)
            if request.method == 'POST':
                branch.branch_name = request.POST.get('branch_name')
                branch.contact_person = request.POST.get('contact_person')
                branch.address = request.POST.get('address')
                branch.city = request.POST.get('city')
                branch.state = request.POST.get('state')
                branch.postal_code = request.POST.get('postal_code')
                branch.country = request.POST.get('country')
                branch.phone = request.POST.get('phone')
                branch.email = request.POST.get('email')
                branch.save()
                messages.success(request, 'Branch updated successfully.')
                return redirect('backoffice:branches')
            return render(request, 'backoffice/branch/edit_branch.html', {'branch': branch})
        except ObjectDoesNotExist:
            messages.error(request, "Branch not found.")
        except IntegrityError as e:
            messages.error(request, f"Data integrity error: {str(e)}")
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:branches')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")

@login_required
def delete_branch(request, branch_id):
    if request.user.is_admin_user or request.user.is_super_user:
        try:
            branch = get_object_or_404(CompanyBranch, id=branch_id)
            branch_name = branch.branch_name
            branch.delete()
            messages.success(request, f"{branch_name} has been deleted successfully.")
        except ObjectDoesNotExist:
            messages.error(request, "Branch not found.")
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:branches')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")